##************************************************************
##
## Example script to handle a user-defined resource
##
## Script takes one parameter, namely the resource name
## specified in the SAS Workload Orchestrator configuration
##
##************************************************************

Param(
   [Parameter(Mandatory=$true, HelpMessage="Resource name as defined in the SAS Workload Orchestrator configuration.")]
   [string] $resName
)


##************************************************************
##
## Get-FreeSpace returns the megabytes free for a specified
## path
##
##************************************************************
function Get-FreeSpace
{
    param (
      $path
    );

    $free = gwmi Win32_Volume -Filter "DriveType=3" `
              | where { $path -Like "$($_.Name)*" } `
              | sort Name -Desc `
              | select -First 1 FreeSpace `
              | % { $_.freespace / (1024*1024) }

    return ([int]$free)
}

##************************************************************
##
## Do something different based on which resource this is
## being asked about
##
##************************************************************
switch($resName)
{
    ##********************************************************
    ## "My Global Static" resource. This is called on the
    ## master when it becomes the master.
    ##
    ## In this example, assume the resource will be the number
    ## of licensed of some product that can be used anywhere
    ## on the grid. We are indicating there are 20 of these
    ## licenses that can be consumed by all jobs on all hosts
    ##********************************************************
    'myGlobalStatic'
    {
        Write-Host "20.0" -NoNewLine
    };

    ##********************************************************
    ## "My Host Static" resource. This is called on the host
    ## whenever it announces itself to the master or when the
    ##
    ## In this example, assume the resource will be the numer
    ## of GPUs available on this host. We are indicating there
    ## are 4 GPUs on this machine that can be consumed by all
    ## jobs routed to this host.
    ##********************************************************
    'myHostStatic'
    {
        Write-Host "4.0" -NoNewLine
    };

    ##********************************************************
    ## "My Host Dynamic" resource. This is called on the host
    ## periodically as defined by the hostUpdateSecs field in
    ## the configuration.
    ##
    ## In this example, assume the resource is the percent of
    ## used space in the SASWORK directory. We are going to
    ## return a value that will be able to be used for
    ## scheduling or suspend thresholds and for sort order.
    ##********************************************************
    'myHostDynamic'
    {
        $freeSpace = Get-FreeSpace("drive:\path\to\SASWORK")
        Write-Host $freeSpace -NoNewLine
    }

    ##********************************************************
    ## Unknown resource. Return 0
    ##********************************************************
    default
    {
        Write-Host "0.0" -NoNewLine
    }
}

