##************************************************************
##
## Example script to handle a service
##
##************************************************************

Param(
   [Parameter(Mandatory=$true, HelpMessage="Action to take on service. Valid values are start, stop, status, or restart.")]
   [string] $action
)

##************************************************************
##
## Some definitions...
##
##************************************************************
$name="SampleService"
$script=$MyInvocation.MyCommand.Name
$now=Get-Date -Format u
$exe="sleep";
$args={"60"};
$thisHost=[System.Net.Dns]::GetHostName();
$tmp=$Env:TEMP + "\";
$namebase=$tmp + $name + "." + $thisHost;
$logFilename=$namebase+".log";
$errFilename=$namebase+".err";
$pidFilename=$namebase+".pid";


##************************************************************
##
## Service function definitions
##
##************************************************************
#
##**********************************************************
## Start Service function
##**********************************************************
function Start-Service
{
   param([string]   $now,
         [string]   $name,
         [string]   $cmd,
         [string[]] $cmdArgs,
         [string]   $logFileName,
         [string]   $errFilename)

   $app = Start-Process -FilePath $cmd `
                        -ArgumentList $cmdArgs `
                        -RedirectStandardOutput $logFilename `
                        -RedirectStandardError $errFilename `
                        -PassThru `
                        -ErrorVariable myErr `
                        -ErrorAction SilentlyContinue;

   if ($app)
   {
      $app.Id | Out-File $pidFilename;

      Write-Host "$now Started service '$name' with process ID $app.Id"

      $errVal = 0;
   }
   else
   {
      Write-Host "$now Service '$name' failed to start"

      $myErr;

      $errVal = 1;
   }

   return($errVal);
}

##**********************************************************
## Stop Service function
##**********************************************************
function Stop-Service
{
   param([string] $now,
         [string] $name,
         [string] $pidFilename)

   $myPid = Get-Content $pidFilename `
                        -ErrorVariable myErr `
                        -ErrorAction SilentlyContinue;

   if (!$myErr)
   {
      Stop-Process -Id $myPid -ErrorAction SilentlyContinue;

      Write-Host "$now Service '$name' with ID $myPid has been stopped"

      Remove-Item $pidFilename;

      $errVal = 0;
   }
   else
   {
      Write-Host "$now Service '$name' is not running"

      $errVal = 1;
   }

   return ($errVal);
}


##************************************************************
##
## Main action handler
##
##************************************************************
switch ($action)
{
  ##**********************************************************
  ## Start service
  ##**********************************************************
  'start'
  {
     $exitCode = Start-Service -now $now `
                               -name $name `
                               -cmd $exe `
                               -cmdArgs $args `
                               -logFilename $logFilename `
                               -errFilename $errFilename;
  };

  ##**********************************************************
  ## Stop service
  ##**********************************************************
  'stop'
  {
     $exitCode = Stop-Service -now $now `
                              -name $name `
                              -pidFilename $pidFilename;
  };

  ##**********************************************************
  ## Get service status
  ##   status = 0, everything is OK
  ##   status < 0, temp error, retry 5 times before restarting
  ##   status > 0, error, try restarting
  ##**********************************************************
  'status'
  {
     $myPid = Get-Content $pidFilename -ErrorVariable myErr -ErrorAction SilentlyContinue;

     if (!$myErr)
     {
         $app = Get-Process -Id $myPid -ErrorVariable myErr -ErrorAction SilentlyContinue;

         if (!$app)
         {
             Write-Host "$now Service '$name' with ID $myPid is not running"

             $exitCode = 1
         }
         else
         {
             Write-Host "$now Service '$name' with ID $myPid is running"

             $exitCode = 0
         }
     }
     else
     {
         Write-Output "$now Service '$name' is not running"

         $exitCode = 1
     }
  };

  ##**********************************************************
  ## Restart service
  ##**********************************************************
  'restart'
  {
     Stop-Service -name $name -pidFilename $pidFilename;

     sleep 1;

     $exitCode = Start-Service -name $name -exe $exe -args $args -logFilename $logFilename -errFilename $errFilename;
  };

  ##**********************************************************
  ## Bad input
  ##**********************************************************
  default
  {
     Write-Host "Invalid option: '$action'";
     Write-Host "Usage: $script [-action] {start|stop|status|restart}";
     $exitCode = 2
  }
}

Write-Host "Exiting with rc=$exitCode"

exit $exitCode
